// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_

#include <string.h>
#include <stdint.h>

#include "third_party/blink/renderer/core/css/parser/css_parser_mode.h"

namespace blink {

enum class CSSValueID {
  kInvalid = 0,
  kInherit = 1,
  kInitial = 2,
  kNone = 3,
  kHidden = 4,
  kInset = 5,
  kGroove = 6,
  kOutset = 7,
  kRidge = 8,
  kDotted = 9,
  kDashed = 10,
  kSolid = 11,
  kDouble = 12,
  kCaption = 13,
  kIcon = 14,
  kMenu = 15,
  kMessageBox = 16,
  kSmallCaption = 17,
  kWebkitMiniControl = 18,
  kWebkitSmallControl = 19,
  kWebkitControl = 20,
  kStatusBar = 21,
  kItalic = 22,
  kOblique = 23,
  kAll = 24,
  kCommonLigatures = 25,
  kNoCommonLigatures = 26,
  kDiscretionaryLigatures = 27,
  kNoDiscretionaryLigatures = 28,
  kHistoricalLigatures = 29,
  kNoHistoricalLigatures = 30,
  kContextual = 31,
  kNoContextual = 32,
  kSmallCaps = 33,
  kAllSmallCaps = 34,
  kPetiteCaps = 35,
  kAllPetiteCaps = 36,
  kUnicase = 37,
  kTitlingCaps = 38,
  kJis78 = 39,
  kJis83 = 40,
  kJis90 = 41,
  kJis04 = 42,
  kSimplified = 43,
  kTraditional = 44,
  kFullWidth = 45,
  kProportionalWidth = 46,
  kRuby = 47,
  kLiningNums = 48,
  kOldstyleNums = 49,
  kProportionalNums = 50,
  kTabularNums = 51,
  kDiagonalFractions = 52,
  kStackedFractions = 53,
  kOrdinal = 54,
  kSlashedZero = 55,
  kNormal = 56,
  kBold = 57,
  kBolder = 58,
  kLighter = 59,
  kUltraCondensed = 60,
  kExtraCondensed = 61,
  kCondensed = 62,
  kSemiCondensed = 63,
  kSemiExpanded = 64,
  kExpanded = 65,
  kExtraExpanded = 66,
  kUltraExpanded = 67,
  kXxSmall = 68,
  kXSmall = 69,
  kSmall = 70,
  kMedium = 71,
  kLarge = 72,
  kXLarge = 73,
  kXxLarge = 74,
  kWebkitXxxLarge = 75,
  kSmaller = 76,
  kLarger = 77,
  kSerif = 78,
  kSansSerif = 79,
  kCursive = 80,
  kFantasy = 81,
  kMonospace = 82,
  kWebkitBody = 83,
  kWebkitPictograph = 84,
  kSwap = 85,
  kFallback = 86,
  kOptional = 87,
  kAqua = 88,
  kBlack = 89,
  kBlue = 90,
  kFuchsia = 91,
  kGray = 92,
  kGreen = 93,
  kLime = 94,
  kMaroon = 95,
  kNavy = 96,
  kOlive = 97,
  kOrange = 98,
  kPurple = 99,
  kRed = 100,
  kSilver = 101,
  kTeal = 102,
  kWhite = 103,
  kYellow = 104,
  kTransparent = 105,
  kWebkitLink = 106,
  kWebkitActivelink = 107,
  kActiveborder = 108,
  kActivecaption = 109,
  kAppworkspace = 110,
  kBackground = 111,
  kButtonface = 112,
  kButtonhighlight = 113,
  kButtonshadow = 114,
  kButtontext = 115,
  kCaptiontext = 116,
  kGraytext = 117,
  kHighlight = 118,
  kHighlighttext = 119,
  kInactiveborder = 120,
  kInactivecaption = 121,
  kInactivecaptiontext = 122,
  kInfobackground = 123,
  kInfotext = 124,
  kMenutext = 125,
  kScrollbar = 126,
  kThreeddarkshadow = 127,
  kThreedface = 128,
  kThreedhighlight = 129,
  kThreedlightshadow = 130,
  kThreedshadow = 131,
  kWindow = 132,
  kWindowframe = 133,
  kWindowtext = 134,
  kInternalActiveListBoxSelection = 135,
  kInternalActiveListBoxSelectionText = 136,
  kInternalInactiveListBoxSelection = 137,
  kInternalInactiveListBoxSelectionText = 138,
  kWebkitFocusRingColor = 139,
  kCurrentcolor = 140,
  kGrey = 141,
  kInternalQuirkInherit = 142,
  kInternalRootColor = 143,
  kRepeat = 144,
  kRepeatX = 145,
  kRepeatY = 146,
  kNoRepeat = 147,
  kClear = 148,
  kCopy = 149,
  kSourceOver = 150,
  kSourceIn = 151,
  kSourceOut = 152,
  kSourceAtop = 153,
  kDestinationOver = 154,
  kDestinationIn = 155,
  kDestinationOut = 156,
  kDestinationAtop = 157,
  kXor = 158,
  kPlusLighter = 159,
  kBaseline = 160,
  kMiddle = 161,
  kSub = 162,
  kSuper = 163,
  kTextTop = 164,
  kTextBottom = 165,
  kTop = 166,
  kBottom = 167,
  kWebkitBaselineMiddle = 168,
  kWebkitAuto = 169,
  kLeft = 170,
  kRight = 171,
  kInlineStart = 172,
  kInlineEnd = 173,
  kCenter = 174,
  kJustify = 175,
  kWebkitLeft = 176,
  kWebkitRight = 177,
  kWebkitCenter = 178,
  kWebkitMatchParent = 179,
  kInternalCenter = 180,
  kInterWord = 181,
  kDistribute = 182,
  kOutside = 183,
  kInside = 184,
  kDisc = 185,
  kCircle = 186,
  kSquare = 187,
  kDecimal = 188,
  kDecimalLeadingZero = 189,
  kArabicIndic = 190,
  kBengali = 191,
  kCambodian = 192,
  kKhmer = 193,
  kDevanagari = 194,
  kGujarati = 195,
  kGurmukhi = 196,
  kKannada = 197,
  kLao = 198,
  kMalayalam = 199,
  kMongolian = 200,
  kMyanmar = 201,
  kOriya = 202,
  kPersian = 203,
  kUrdu = 204,
  kTelugu = 205,
  kTibetan = 206,
  kThai = 207,
  kLowerRoman = 208,
  kUpperRoman = 209,
  kLowerGreek = 210,
  kLowerAlpha = 211,
  kLowerLatin = 212,
  kUpperAlpha = 213,
  kUpperLatin = 214,
  kCjkEarthlyBranch = 215,
  kCjkHeavenlyStem = 216,
  kEthiopicHalehame = 217,
  kEthiopicHalehameAm = 218,
  kEthiopicHalehameTiEr = 219,
  kEthiopicHalehameTiEt = 220,
  kHangul = 221,
  kHangulConsonant = 222,
  kKoreanHangulFormal = 223,
  kKoreanHanjaFormal = 224,
  kKoreanHanjaInformal = 225,
  kHebrew = 226,
  kArmenian = 227,
  kLowerArmenian = 228,
  kUpperArmenian = 229,
  kGeorgian = 230,
  kCjkIdeographic = 231,
  kSimpChineseFormal = 232,
  kSimpChineseInformal = 233,
  kTradChineseFormal = 234,
  kTradChineseInformal = 235,
  kHiragana = 236,
  kKatakana = 237,
  kHiraganaIroha = 238,
  kKatakanaIroha = 239,
  kInline = 240,
  kBlock = 241,
  kFlowRoot = 242,
  kListItem = 243,
  kInlineBlock = 244,
  kTable = 245,
  kInlineTable = 246,
  kTableRowGroup = 247,
  kTableHeaderGroup = 248,
  kTableFooterGroup = 249,
  kTableRow = 250,
  kTableColumnGroup = 251,
  kTableColumn = 252,
  kTableCell = 253,
  kTableCaption = 254,
  kWebkitBox = 255,
  kWebkitInlineBox = 256,
  kFlex = 257,
  kInlineFlex = 258,
  kGrid = 259,
  kInlineGrid = 260,
  kContents = 261,
  kWebkitFlex = 262,
  kWebkitInlineFlex = 263,
  kLayout = 264,
  kInlineLayout = 265,
  kAuto = 266,
  kCrosshair = 267,
  kDefault = 268,
  kPointer = 269,
  kMove = 270,
  kVerticalText = 271,
  kCell = 272,
  kContextMenu = 273,
  kAlias = 274,
  kProgress = 275,
  kNoDrop = 276,
  kNotAllowed = 277,
  kZoomIn = 278,
  kZoomOut = 279,
  kEResize = 280,
  kNeResize = 281,
  kNwResize = 282,
  kNResize = 283,
  kSeResize = 284,
  kSwResize = 285,
  kSResize = 286,
  kWResize = 287,
  kEwResize = 288,
  kNsResize = 289,
  kNeswResize = 290,
  kNwseResize = 291,
  kColResize = 292,
  kRowResize = 293,
  kText = 294,
  kWait = 295,
  kHelp = 296,
  kAllScroll = 297,
  kGrab = 298,
  kGrabbing = 299,
  kWebkitGrab = 300,
  kWebkitGrabbing = 301,
  kWebkitZoomIn = 302,
  kWebkitZoomOut = 303,
  kLtr = 304,
  kRtl = 305,
  kCapitalize = 306,
  kUppercase = 307,
  kLowercase = 308,
  kVisible = 309,
  kCollapse = 310,
  kA3 = 311,
  kA4 = 312,
  kA5 = 313,
  kAbove = 314,
  kAbsolute = 315,
  kAlways = 316,
  kAvoid = 317,
  kB4 = 318,
  kB5 = 319,
  kBelow = 320,
  kBidiOverride = 321,
  kBlink = 322,
  kBoth = 323,
  kBreakSpaces = 324,
  kCloseQuote = 325,
  kEmbed = 326,
  kFixed = 327,
  kHand = 328,
  kHide = 329,
  kIsolate = 330,
  kIsolateOverride = 331,
  kPlaintext = 332,
  kWebkitIsolate = 333,
  kWebkitIsolateOverride = 334,
  kWebkitPlaintext = 335,
  kLandscape = 336,
  kLedger = 337,
  kLegal = 338,
  kLetter = 339,
  kLineThrough = 340,
  kLocal = 341,
  kNoCloseQuote = 342,
  kNoOpenQuote = 343,
  kNowrap = 344,
  kOpenQuote = 345,
  kOverlay = 346,
  kOverline = 347,
  kPortrait = 348,
  kPre = 349,
  kPreLine = 350,
  kPreWrap = 351,
  kRelative = 352,
  kScroll = 353,
  kSeparate = 354,
  kShow = 355,
  kStatic = 356,
  kThick = 357,
  kThin = 358,
  kUnderline = 359,
  kWavy = 360,
  kWebkitNowrap = 361,
  kStretch = 362,
  kStart = 363,
  kEnd = 364,
  kClone = 365,
  kSlice = 366,
  kReverse = 367,
  kHorizontal = 368,
  kVertical = 369,
  kInlineAxis = 370,
  kBlockAxis = 371,
  kFlexStart = 372,
  kFlexEnd = 373,
  kSpaceBetween = 374,
  kSpaceAround = 375,
  kSpaceEvenly = 376,
  kUnsafe = 377,
  kSafe = 378,
  kRow = 379,
  kRowReverse = 380,
  kColumn = 381,
  kColumnReverse = 382,
  kWrap = 383,
  kWrapReverse = 384,
  kAutoFlow = 385,
  kDense = 386,
  kReadOnly = 387,
  kReadWrite = 388,
  kReadWritePlaintextOnly = 389,
  kElement = 390,
  kWebkitMinContent = 391,
  kWebkitMaxContent = 392,
  kWebkitFillAvailable = 393,
  kWebkitFitContent = 394,
  kMinContent = 395,
  kMaxContent = 396,
  kFitContent = 397,
  kClip = 398,
  kEllipsis = 399,
  kDiscard = 400,
  kBreakAll = 401,
  kKeepAll = 402,
  kBreakWord = 403,
  kSpace = 404,
  kLoose = 405,
  kStrict = 406,
  kAfterWhiteSpace = 407,
  kAnywhere = 408,
  kManual = 409,
  kCheckbox = 410,
  kRadio = 411,
  kPushButton = 412,
  kSquareButton = 413,
  kButton = 414,
  kInnerSpinButton = 415,
  kListbox = 416,
  kMediaSlider = 417,
  kMediaSliderthumb = 418,
  kMediaVolumeSlider = 419,
  kMediaVolumeSliderthumb = 420,
  kInternalMediaControl = 421,
  kMenulist = 422,
  kMenulistButton = 423,
  kMeter = 424,
  kProgressBar = 425,
  kSliderHorizontal = 426,
  kSliderVertical = 427,
  kSliderthumbHorizontal = 428,
  kSliderthumbVertical = 429,
  kSearchfield = 430,
  kSearchfieldCancelButton = 431,
  kTextfield = 432,
  kTextarea = 433,
  kRound = 434,
  kBorder = 435,
  kBorderBox = 436,
  kContent = 437,
  kContentBox = 438,
  kPadding = 439,
  kPaddingBox = 440,
  kMarginBox = 441,
  kContain = 442,
  kCover = 443,
  kLogical = 444,
  kVisual = 445,
  kAlternate = 446,
  kAlternateReverse = 447,
  kForwards = 448,
  kBackwards = 449,
  kInfinite = 450,
  kRunning = 451,
  kPaused = 452,
  kFlat = 453,
  kPreserve3d = 454,
  kFillBox = 455,
  kViewBox = 456,
  kEase = 457,
  kLinear = 458,
  kEaseIn = 459,
  kEaseOut = 460,
  kEaseInOut = 461,
  kJumpBoth = 462,
  kJumpEnd = 463,
  kJumpNone = 464,
  kJumpStart = 465,
  kStepStart = 466,
  kStepEnd = 467,
  kSteps = 468,
  kFrames = 469,
  kCubicBezier = 470,
  kDocument = 471,
  kReset = 472,
  kZoom = 473,
  kVisiblepainted = 474,
  kVisiblefill = 475,
  kVisiblestroke = 476,
  kPainted = 477,
  kFill = 478,
  kStroke = 479,
  kBoundingBox = 480,
  kSpellOut = 481,
  kDigits = 482,
  kLiteralPunctuation = 483,
  kNoPunctuation = 484,
  kAntialiased = 485,
  kSubpixelAntialiased = 486,
  kOptimizespeed = 487,
  kOptimizelegibility = 488,
  kGeometricprecision = 489,
  kCrispedges = 490,
  kEconomy = 491,
  kExact = 492,
  kLr = 493,
  kRl = 494,
  kTb = 495,
  kLrTb = 496,
  kRlTb = 497,
  kTbRl = 498,
  kHorizontalTb = 499,
  kVerticalRl = 500,
  kVerticalLr = 501,
  kAfter = 502,
  kBefore = 503,
  kOver = 504,
  kUnder = 505,
  kFilled = 506,
  kOpen = 507,
  kDot = 508,
  kDoubleCircle = 509,
  kTriangle = 510,
  kSesame = 511,
  kEllipse = 512,
  kClosestSide = 513,
  kClosestCorner = 514,
  kFarthestSide = 515,
  kFarthestCorner = 516,
  kMixed = 517,
  kSideways = 518,
  kSidewaysRight = 519,
  kUpright = 520,
  kVerticalRight = 521,
  kOn = 522,
  kOff = 523,
  kOptimizequality = 524,
  kPixelated = 525,
  kWebkitOptimizeContrast = 526,
  kFromImage = 527,
  kNonzero = 528,
  kEvenodd = 529,
  kAt = 530,
  kAlphabetic = 531,
  kFullscreen = 532,
  kStandalone = 533,
  kMinimalUi = 534,
  kBrowser = 535,
  kSticky = 536,
  kCoarse = 537,
  kFine = 538,
  kOnDemand = 539,
  kHover = 540,
  kMultiply = 541,
  kScreen = 542,
  kDarken = 543,
  kLighten = 544,
  kColorDodge = 545,
  kColorBurn = 546,
  kHardLight = 547,
  kSoftLight = 548,
  kDifference = 549,
  kExclusion = 550,
  kHue = 551,
  kSaturation = 552,
  kColor = 553,
  kLuminosity = 554,
  kScaleDown = 555,
  kBalance = 556,
  kDrag = 557,
  kNoDrag = 558,
  kSpan = 559,
  kMinmax = 560,
  kEachLine = 561,
  kProgressive = 562,
  kInterlace = 563,
  kMarkers = 564,
  kInternalExtendToZoom = 565,
  kPanX = 566,
  kPanY = 567,
  kPanLeft = 568,
  kPanRight = 569,
  kPanUp = 570,
  kPanDown = 571,
  kManipulation = 572,
  kPinchZoom = 573,
  kLastBaseline = 574,
  kFirstBaseline = 575,
  kFirst = 576,
  kLast = 577,
  kSelfStart = 578,
  kSelfEnd = 579,
  kLegacy = 580,
  kSmooth = 581,
  kScrollPosition = 582,
  kRevert = 583,
  kUnset = 584,
  kLinearGradient = 585,
  kRadialGradient = 586,
  kConicGradient = 587,
  kRepeatingLinearGradient = 588,
  kRepeatingRadialGradient = 589,
  kRepeatingConicGradient = 590,
  kPaint = 591,
  kWebkitCrossFade = 592,
  kWebkitGradient = 593,
  kWebkitLinearGradient = 594,
  kWebkitRadialGradient = 595,
  kWebkitRepeatingLinearGradient = 596,
  kWebkitRepeatingRadialGradient = 597,
  kWebkitImageSet = 598,
  kFrom = 599,
  kTo = 600,
  kColorStop = 601,
  kRadial = 602,
  kAttr = 603,
  kCounter = 604,
  kCounters = 605,
  kRect = 606,
  kPolygon = 607,
  kFormat = 608,
  kInvert = 609,
  kGrayscale = 610,
  kSepia = 611,
  kSaturate = 612,
  kHueRotate = 613,
  kOpacity = 614,
  kBrightness = 615,
  kContrast = 616,
  kBlur = 617,
  kDropShadow = 618,
  kUrl = 619,
  kRgb = 620,
  kRgba = 621,
  kHsl = 622,
  kHsla = 623,
  kMatrix = 624,
  kMatrix3d = 625,
  kPerspective = 626,
  kRotate = 627,
  kRotateX = 628,
  kRotateY = 629,
  kRotateZ = 630,
  kRotate3d = 631,
  kScale = 632,
  kScaleX = 633,
  kScaleY = 634,
  kScaleZ = 635,
  kScale3d = 636,
  kSkew = 637,
  kSkewX = 638,
  kSkewY = 639,
  kTranslate = 640,
  kTranslateX = 641,
  kTranslateY = 642,
  kTranslateZ = 643,
  kTranslate3d = 644,
  kX = 645,
  kY = 646,
  kZ = 647,
  kPath = 648,
  kRay = 649,
  kSides = 650,
  kCalc = 651,
  kWebkitCalc = 652,
  kMin = 653,
  kMax = 654,
  kClamp = 655,
  kMandatory = 656,
  kProximity = 657,
  kStyle = 658,
  kSize = 659,
  kAutoFill = 660,
  kAutoFit = 661,
  kVar = 662,
  kInternalVariableValue = 663,
  kEnv = 664,
  kAvoidPage = 665,
  kPage = 666,
  kRecto = 667,
  kVerso = 668,
  kAvoidColumn = 669,
  kP3 = 670,
  kRec2020 = 671,
  kTrue = 672,
  kFalse = 673,
  kNoPreference = 674,
  kDark = 675,
  kLight = 676,
  kOnly = 677,
  kReduce = 678,
  kActive = 679,
  kAliceblue = 680,
  kAntiquewhite = 681,
  kAquamarine = 682,
  kAzure = 683,
  kBeige = 684,
  kBisque = 685,
  kBlanchedalmond = 686,
  kBlueviolet = 687,
  kBrown = 688,
  kBurlywood = 689,
  kCadetblue = 690,
  kChartreuse = 691,
  kChocolate = 692,
  kCoral = 693,
  kCornflowerblue = 694,
  kCornsilk = 695,
  kCrimson = 696,
  kCyan = 697,
  kDarkblue = 698,
  kDarkcyan = 699,
  kDarkgoldenrod = 700,
  kDarkgray = 701,
  kDarkgreen = 702,
  kDarkgrey = 703,
  kDarkkhaki = 704,
  kDarkmagenta = 705,
  kDarkolivegreen = 706,
  kDarkorange = 707,
  kDarkorchid = 708,
  kDarkred = 709,
  kDarksalmon = 710,
  kDarkseagreen = 711,
  kDarkslateblue = 712,
  kDarkslategray = 713,
  kDarkslategrey = 714,
  kDarkturquoise = 715,
  kDarkviolet = 716,
  kDeeppink = 717,
  kDeepskyblue = 718,
  kDimgray = 719,
  kDimgrey = 720,
  kDodgerblue = 721,
  kFirebrick = 722,
  kFloralwhite = 723,
  kForestgreen = 724,
  kGainsboro = 725,
  kGhostwhite = 726,
  kGold = 727,
  kGoldenrod = 728,
  kGreenyellow = 729,
  kHoneydew = 730,
  kHotpink = 731,
  kIndianred = 732,
  kIndigo = 733,
  kIvory = 734,
  kKhaki = 735,
  kLavender = 736,
  kLavenderblush = 737,
  kLawngreen = 738,
  kLemonchiffon = 739,
  kLightblue = 740,
  kLightcoral = 741,
  kLightcyan = 742,
  kLightgoldenrodyellow = 743,
  kLightgray = 744,
  kLightgreen = 745,
  kLightgrey = 746,
  kLightpink = 747,
  kLightsalmon = 748,
  kLightseagreen = 749,
  kLightskyblue = 750,
  kLightslategray = 751,
  kLightslategrey = 752,
  kLightsteelblue = 753,
  kLightyellow = 754,
  kLimegreen = 755,
  kLinen = 756,
  kMagenta = 757,
  kMediumaquamarine = 758,
  kMediumblue = 759,
  kMediumorchid = 760,
  kMediumpurple = 761,
  kMediumseagreen = 762,
  kMediumslateblue = 763,
  kMediumspringgreen = 764,
  kMediumturquoise = 765,
  kMediumvioletred = 766,
  kMidnightblue = 767,
  kMintcream = 768,
  kMistyrose = 769,
  kMoccasin = 770,
  kNavajowhite = 771,
  kOldlace = 772,
  kOlivedrab = 773,
  kOrangered = 774,
  kOrchid = 775,
  kPalegoldenrod = 776,
  kPalegreen = 777,
  kPaleturquoise = 778,
  kPalevioletred = 779,
  kPapayawhip = 780,
  kPeachpuff = 781,
  kPeru = 782,
  kPink = 783,
  kPlum = 784,
  kPowderblue = 785,
  kRebeccapurple = 786,
  kRosybrown = 787,
  kRoyalblue = 788,
  kSaddlebrown = 789,
  kSalmon = 790,
  kSandybrown = 791,
  kSeagreen = 792,
  kSeashell = 793,
  kSienna = 794,
  kSkyblue = 795,
  kSlateblue = 796,
  kSlategray = 797,
  kSlategrey = 798,
  kSnow = 799,
  kSpringgreen = 800,
  kSteelblue = 801,
  kTan = 802,
  kThistle = 803,
  kTomato = 804,
  kTurquoise = 805,
  kViolet = 806,
  kWheat = 807,
  kWhitesmoke = 808,
  kYellowgreen = 809,
  kAlpha = 810,
  kLuminance = 811,
  kAccumulate = 812,
  kNew = 813,
  kSRGB = 814,
  kLinearrgb = 815,
  kButt = 816,
  kMiter = 817,
  kBevel = 818,
  kBeforeEdge = 819,
  kAfterEdge = 820,
  kCentral = 821,
  kTextBeforeEdge = 822,
  kTextAfterEdge = 823,
  kIdeographic = 824,
  kHanging = 825,
  kMathematical = 826,
  kUseScript = 827,
  kNoChange = 828,
  kResetSize = 829,
  kDynamic = 830,
  kNonScalingStroke = 831,
};

const int numCSSValueKeywords = 832;
const size_t maxCSSValueKeywordLength = 42;

inline bool IsValidCSSValueID(CSSValueID id)
{
    return id != CSSValueID::kInvalid;
}

const char* getValueName(CSSValueID);
bool isValueAllowedInMode(CSSValueID id, CSSParserMode mode);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
